from enum import Enum
from pathlib import Path

import typer
from tabulate import tabulate

from .api import create_api
from .commons import parse_csv, parse_json, parse_jsonlines
from .exceptions import cli_wrapper

app = typer.Typer()
token_app = typer.Typer(help="Manage API Tokens.")
app.add_typer(token_app, name="token")


@app.command("create")
@cli_wrapper
def create_bucket(
    description: str = typer.Option("", show_default=False),
    schemaless: bool = typer.Option(
        False,
        "--schemaless",
        help="Create a schemaless bucket.",
        show_default=False,
    ),
):
    """
    Create a new bucket.
    """
    api = create_api()
    payload = api.Buckets.create(description, schemaless=schemaless)
    typer.echo(
        typer.style(
            "Bucket created, you can start sending data 🚀",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )
    first_row = ("uuid", "name", "description", "url")
    values = [getattr(payload, entry) for entry in first_row]
    first_row = ("ID", "Name", "Description", "URL")
    typer.echo(
        tabulate(
            [first_row, values], headers="firstrow", tablefmt="fancy_grid"
        )
    )


@app.command("list")
@cli_wrapper
def list_buckets():
    """
    Get a list of your buckets.
    """
    api = create_api()
    payload = api.Buckets.list()
    first_row = ("uuid", "name", "description", "url")

    if not payload:
        typer.echo(
            typer.style(
                "Ohhhh... no buckets :)", fg=typer.colors.MAGENTA, bold=True
            )
        )
        return

    rows = []
    for index, row in enumerate(payload, 1):
        rows.append([index, *[getattr(row, entry) for entry in first_row]])

    first_row = ("ID", "Name", "Description", "URL", "Schemaless")
    first_row = ("", *first_row)
    typer.echo(
        tabulate([first_row, *rows], headers="firstrow", tablefmt="fancy_grid")
    )


@app.command("delete")
@cli_wrapper
def delete_bucket(bucket: str):
    """
    Delete a bucket.
    """
    api = create_api()
    api.Buckets.delete(bucket)
    typer.echo(
        typer.style(
            f"Bucket {bucket} deleted.", fg=typer.colors.WHITE, bold=True
        )
    )


@app.command("query")
@cli_wrapper
def query_bucket(query: str = typer.Argument(...)):
    """
    Query a bucket using a SQL query.
    """
    api = create_api()
    payload = api.Buckets.query(query)
    for row in payload:
        typer.echo(row)


@token_app.command("create")
@cli_wrapper
def create_token(
    bucket: str = typer.Argument(
        ..., help="The bucket uuid you want to give access to."
    ),
    description: str = typer.Option(""),
):
    """
    Create a new API Token.
    """
    api = create_api()
    client = api.Bucket(bucket)
    payload = client.create_token(description)
    typer.echo(
        typer.style(
            "Bucket Token created.",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )
    typer.echo(
        typer.style(
            "⚠️  PLEASE SAVE THE TOKEN, THIS IS THE ONLY TIME WHERE WE DISPLAY IT ⚠️",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )
    first_row = ("uuid", "token", "description", "created_on")
    values = [payload[entry] for entry in first_row]
    first_row = ("ID", "Token", "Description", "Created on")
    typer.echo(
        tabulate(
            [first_row, values], headers="firstrow", tablefmt="fancy_grid"
        )
    )


@token_app.command("list")
@cli_wrapper
def list_tokens(bucket: str = typer.Argument(..., help="The bucket uuid.")):
    """
    Get your tokens.
    """
    api = create_api()
    client = api.Bucket(bucket)
    tokens = client.get_tokens()
    first_row = ("uuid", "token", "description", "created_on")
    if not tokens:
        typer.echo(
            typer.style(
                f"No tokens for Bucket({bucket}).",
                fg=typer.colors.WHITE,
                bold=True,
            )
        )
        return

    rows = []
    for index, row in enumerate(tokens, 1):
        rows.append([index, *[row[entry] for entry in first_row]])

    first_row = ("ID", "Token", "Description", "Created on")
    first_row = ("", *first_row)
    typer.echo(
        tabulate([first_row, *rows], headers="firstrow", tablefmt="fancy_grid")
    )


@token_app.command("delete")
@cli_wrapper
def delete_token(bucket_uuid: str, token_uuid: str):
    """
    Delete an API Token.
    """
    api = create_api()
    client = api.Bucket(bucket_uuid)
    client.delete_token(token_uuid=token_uuid)
    typer.echo(
        typer.style(
            f"Bucket Token {token_uuid} deleted.",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )


class DataFileFormat(str, Enum):
    csv = "csv"
    json = "json"
    jsonlines = "jsonlines"


@app.command("push")
@cli_wrapper
def push_data(
    bucket_uuid: str,
    file: Path = typer.Option(
        ...,
        help="The path to the data file",
        exists=True,
        file_okay=True,
        dir_okay=False,
        writable=False,
        readable=True,
        resolve_path=True,
    ),
    format: DataFileFormat = typer.Option(
        ..., help="The format of the data file", case_sensitive=False
    ),
):
    """
    Push the content of a data file into the bucket.
    The file can be CSV, JSON, or JSON-lines.
    """
    if format == DataFileFormat.csv:
        rows = parse_csv(file)
    elif format == DataFileFormat.json:
        rows = parse_json(file)
    elif format == DataFileFormat.csv:
        rows = parse_jsonlines(file)

    api = create_api()
    bucket = api.Bucket(bucket_uuid)
    bucket.push_data(rows=rows)
    typer.echo(
        typer.style(
            "The data has been loaded 🚀.",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )



import typer

from . import commons
from .api import create_api
from .exceptions import cli_wrapper

user = typer.Typer()


@user.command()
@cli_wrapper
def create():
    """
    Create a new user.
    """
    typer.echo("Hey there 👋")

    email = typer.prompt("> What's your email?")
    password = typer.prompt(
        "> What's your password?", hide_input=True, confirmation_prompt=True
    )

    api = create_api(with_auth=False)
    api.User.create(email, password)

    typer.echo(
        typer.style(
            "Awesome, check your email to confirm your email address!",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )


@cli_wrapper
def login_with_email_or_token(
    email: str = typer.Option(None, help="Your user's email."),
    token: str = typer.Option(None, help="Your API or Bucket token."),
):
    """
    Use the cli to log-in (with email/password or api/bucket token).
    """
    if not email and not token:
        typer.echo(
            typer.style(
                "You need to pass your user'email or a token.",
                fg=typer.colors.RED,
                bold=True,
            )
        )
        raise typer.Exit()
    auth = {}
    if email:
        password = typer.prompt("> What's your password?", hide_input=True)
        api = create_api(with_auth=False)
        response = api.User.login(email, password)
        auth["email"] = email
        auth["token"] = response["access_token"]
    elif token:
        auth["token"] = token
    commons.save_auth(auth)


@cli_wrapper
def logout():
    """
    To Logout. For real.
    """
    commons.save_auth({})


@user.command()
@cli_wrapper
def reset_password(email: str = typer.Option(...)):
    """
    Reset your user's password
    """
    api = create_api(with_auth=False)
    api.User.request_password_reset(email)
    typer.echo(
        typer.style(
            "Awesome, check your email for the next steps for the password reset!",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )


@user.command()
@cli_wrapper
def email_verification(email: str = typer.Option(...)):
    """
    Request a new email verification email
    """
    password = typer.prompt("> What's your password?", hide_input=True)
    api = create_api(with_auth=False)
    api.User.request_verification_email(email, password)
    typer.echo(
        typer.style(
            "Awesome, check your email to confirm your email address!",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )



import json
import functools

import typer


class AdacordApiError(Exception):
    def __init__(self, message, status_code):
        if isinstance(message, dict):
            self.message = message.get("message", json.dumps(message))
        else:
            self.message = message

        self.status_code = status_code


def cli_wrapper(func):
    """Catch execpetions and return cli printable errors."""

    @functools.wraps(func)
    def decorator(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except AdacordApiError as err:
            typer.echo(
                typer.style(
                    f"Error: {err.message}", fg=typer.colors.RED, bold=True
                )
            )
        except json.JSONDecodeError:
            typer.echo(
                typer.style(
                    "Ouch.... something bad happened :/",
                    fg=typer.colors.RED,
                    bold=True,
                )
            )

    return decorator



import csv
import json
from typing import Any, Dict, List
from pathlib import Path

CONFIG_FOLDER_PATH = Path.home() / ".adacord"


def save_auth(payload, base_path=CONFIG_FOLDER_PATH):
    Path(base_path).mkdir(exist_ok=True)
    with open(base_path / "auth.json", "w+") as f:
        f.write(json.dumps(payload))


def read_auth(base_path=CONFIG_FOLDER_PATH):
    with open(base_path / "auth.json", "r+") as f:
        return json.loads(f.read())


def get_token(base_path=CONFIG_FOLDER_PATH):
    auth = read_auth(base_path)
    return auth["token"]


def parse_csv(filepath: Path) -> List[Dict[str, Any]]:
    rows = []
    with filepath.open(encoding="utf-8") as csvf:
        csvReader = csv.DictReader(csvf)
        csvReader.fieldnames = [
            field.lower() for field in csvReader.fieldnames
        ]
        rows = [row for row in csvReader]
    return rows


def parse_json(filepath: Path) -> List[Dict[str, Any]]:
    with open(filepath) as f:
        rows = json.load(f)
    if isinstance(rows, dict):
        rows = [rows]
    return rows


def parse_jsonlines(filepath: Path) -> List[Dict[str, Any]]:
    rows = []
    with filepath.open("r") as f:
        for line in f:
            rows.append(json.loads(line))
    return rows



import typer
from tabulate import tabulate

from .api import create_api
from .exceptions import cli_wrapper

app = typer.Typer()


@app.command("create")
@cli_wrapper
def create_api_token(
    description: str = typer.Option(""),
):
    """
    Create a new API Token.
    """
    api = create_api()
    payload = api.ApiTokens.create(description)
    typer.echo(
        typer.style(
            "API Token created.",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )
    typer.echo(
        typer.style(
            "⚠️  PLEASE SAVE THE TOKEN, THIS IS THE ONLY TIME WHERE WE DISPLAY IT ⚠️",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )
    first_row = ("uuid", "token", "description", "created_on")
    values = [payload[entry] for entry in first_row]
    first_row = ("ID", "Token", "Description", "Created on")
    typer.echo(
        tabulate(
            [first_row, values], headers="firstrow", tablefmt="fancy_grid"
        )
    )


@app.command("list")
@cli_wrapper
def list_api_tokens():
    """
    Get your API tokens.
    """
    api = create_api()
    api_tokens = api.ApiTokens.list()
    first_row = ("uuid", "token", "description", "created_on")
    if not api_tokens:
        typer.echo(
            typer.style(
                "No API tokens.",
                fg=typer.colors.WHITE,
                bold=True,
            )
        )
        return

    rows = []
    for index, row in enumerate(api_tokens, 1):
        rows.append([index, *[row[entry] for entry in first_row]])

    first_row = ("ID", "Token", "Description", "Created on")
    first_row = ("", *first_row)
    typer.echo(
        tabulate([first_row, *rows], headers="firstrow", tablefmt="fancy_grid")
    )


@app.command("delete")
@cli_wrapper
def delete_api_token(token_uuid: str):
    """
    Delete an API Token.
    """
    api = create_api()
    api.ApiTokens.delete(token_uuid=token_uuid)
    typer.echo(
        typer.style(
            f"API Token {token_uuid} deleted.",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['adacord', 'adacord.cli']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.26.0,<3.0.0',
 'tabulate>=0.8.9,<0.9.0',
 'typer[all]>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['adacord = adacord.cli.main:app']}

setup_kwargs = {
    'name': 'adacord',
    'version': '0.1.9',
    'description': 'The cli and sdk for adacord.com',
    'long_description': '# Adacord CLI\n\n\n## Installation\n\n```bash\npip install adacord\n```\n\n## Usage\n\n### Create a new user\n\n```bash\nadacord user create\n```\n\n### Login\n\n```bash\nadacord user login --email me@my-email.com --password your-password\n```\n\n### Create endpoint\n\n```bash\nadacord bucket create --description "A fancy bucket"\n```\n\n### List endpoints\n\n```bash\nadacord bucket list\n```\n\n### Query endpoint\n\n```bash\nadacord bucket query my-bucket --query \'select * from my-bucket\'\n```\n\n# Contributing\n\n```bash\npoetry install\n```\n',
    'author': 'Christian Barra',
    'author_email': 'me@christianbarra.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://www.adacord.com',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import typer

from . import auth, bucket, api_tokens

app = typer.Typer()
app.add_typer(auth.user, name="user", help="User related stuff")
# TODO: Maybe there's a better way instead
# of calling the decorator in this way?
app.command("login")(auth.login_with_email_or_token)
app.command("logout")(auth.logout)
app.add_typer(bucket.app, name="bucket", help="Manage your buckets")
app.add_typer(api_tokens.app, name="api_tokens", help="Manage your API tokens")



import urllib
from typing import Any, Dict, List, Union, Callable
from dataclasses import dataclass

import requests
from requests.auth import AuthBase

from .commons import get_token
from .exceptions import AdacordApiError

HTTP_TIMEOUT = 10


class AccessTokenAuth(AuthBase):
    """A class that adds a Bearer token to the request headers."""

    def __init__(self, token_getter: Callable[[], str]):
        self._token_getter = token_getter
        self._token = None

    def get_token(self):

        if not self._token:
            self._token = self._token_getter()
        return self._token

    def __call__(self, request):

        request.headers["Authorization"] = f"Bearer {self.get_token()}"
        return request


class CustomHTTPAdapter(requests.adapters.HTTPAdapter):
    """
    This adapter is a wrapper for the default HTTPAdapter for further
    customization. It does:
        - custom exception raising.
    """

    def send(self, req, *args, **kwargs):
        response = super().send(req, *args, **kwargs)
        try:
            response.raise_for_status()
        except requests.HTTPError as error:
            raise AdacordApiError(
                response.json(), status_code=response.status_code
            ) from error
        else:
            return response


class HTTPClient(requests.Session):
    """A class to make HTTP requests using the CustomHTTPAdapter."""

    def __init__(self, auth: AuthBase = None):
        super().__init__()
        self.auth: AuthBase = auth
        adapter = CustomHTTPAdapter()
        self.mount("http://", adapter)
        self.mount("https://", adapter)

    def request(self, method: str, url: str, *args, **kwargs):
        response = super().request(
            method, url, timeout=HTTP_TIMEOUT, *args, **kwargs
        )
        if not response.ok:
            raise AdacordApiError(
                response.json(), status_code=response.status_code
            )
        return response

    @classmethod
    def with_token(cls, token: str) -> "HTTPClient":
        return cls(auth=AccessTokenAuth(token_getter=lambda: token))


class ApiClient:
    """Client for the Adacord API"""

    def __init__(self, client: HTTPClient):
        self.client = client

    @property
    def base_path(self) -> str:
        return "https://api.adacord.com"

    def url_for(self, endpoint: str, version: str = "v0") -> str:
        """Return the absolute URL to the endpoint for the given API version."""
        suffix = f"{version}/{endpoint}" if endpoint else version
        return urllib.parse.urljoin(self.base_path, suffix)


class User(ApiClient):
    def create(self, email: str, password: str):
        data = {"email": email, "password": password}
        url = self.url_for("/users")
        self.client.post(url, json=data, auth=False)

    def login(self, email: str, password: str) -> Dict[str, Any]:
        data = {"email": email, "password": password}
        url = self.url_for("/users/token")
        response = self.client.post(url, json=data, auth=False)
        return response.json()

    def request_password_reset(self, email: str) -> Dict[str, Any]:
        data = {"email": email}
        url = self.url_for("/users/password_reset")
        response = self.client.post(url, json=data, auth=False)
        return response.json()

    def request_verification_email(
        self, email: str, password: str
    ) -> Dict[str, Any]:
        data = {"email": email, "password": password}
        url = self.url_for("/users/verification_email")
        response = self.client.post(url, json=data, auth=False)
        return response.json()


class Buckets(ApiClient):
    def _bucket_from_payload(
        self, bucket_payload: Dict[str, Any]
    ) -> Union["Bucket", List["Bucket"]]:
        bucket_args = BucketArgs(**bucket_payload)
        return Bucket(bucket_args, buckets_router=self)

    def create(self, description: str, schemaless: bool = False) -> "Bucket":
        data = {"description": description, "schemaless": schemaless}
        url = self.url_for("/buckets")
        response = self.client.post(url, json=data)
        bucket_payload = response.json()
        return self._bucket_from_payload(bucket_payload)

    def list(self) -> List["Bucket"]:
        endpoint = "/buckets"
        url = self.url_for(endpoint)
        response = self.client.get(url)
        bucket_payload = response.json()
        return [
            self._bucket_from_payload(payload) for payload in bucket_payload
        ]

    def get(self, bucket: str) -> "Bucket":
        """Return a Bucket.
        Args:
            bucket: the name or the uuid of the bucket.
        """
        endpoint = f"/buckets/{bucket}"
        url = self.url_for(endpoint)
        response = self.client.get(url)
        bucket_payload = response.json()
        return self._bucket_from_payload(bucket_payload)

    def delete(self, bucket: str) -> Dict[str, Any]:
        url = self.url_for(f"/buckets/{bucket}")
        response = self.client.delete(url)
        return response.json()

    def create_token(self, bucket: str, description: str = None):
        data = {"description": description}
        url = self.url_for(f"/buckets/{bucket}/tokens")
        response = self.client.post(url, json=data)
        return response.json()

    def get_tokens(self, bucket: str):
        url = self.url_for(f"/buckets/{bucket}/tokens")
        response = self.client.get(url)
        return response.json()

    def delete_token(self, bucket: str, token_uuid: str):
        url = self.url_for(f"/buckets/{bucket}/tokens/{token_uuid}")
        response = self.client.delete(url)
        return response.json()

    def query(self, query: str) -> List[Dict[str, Any]]:
        data = {"query": query}
        response = self.client.post(self.url_for("/buckets/query"), json=data)
        return response.json()

    def push_data(self, bucket: str, rows: List[Dict[str, Any]]):
        data = {"data": rows}
        response = self.client.post(
            self.url_for(f"/buckets/{bucket}/data"), json=data
        )
        return response.json()

    def get_data(self, bucket: str) -> List[Dict[str, Any]]:
        response = self.client.get(self.url_for(f"/buckets/{bucket}/data"))
        return response.json()


class ApiTokens(ApiClient):
    def create(self, description: str = None):
        data = {"description": description}
        url = self.url_for("/api_tokens")
        response = self.client.post(url, json=data)
        return response.json()

    def list(self):
        url = self.url_for("/api_tokens")
        response = self.client.get(url)
        return response.json()

    def delete(self, token_uuid: str):
        url = self.url_for(f"/api_tokens/{token_uuid}")
        response = self.client.delete(url)
        return response.json()


@dataclass
class BucketArgs:
    uuid: str
    name: str
    description: str
    url: str
    schemaless: bool


class Bucket:
    def __init__(
        self,
        bucket_payload: BucketArgs,
        buckets_router: "Buckets",
    ):
        self.uuid = bucket_payload.uuid
        self.name = bucket_payload.name
        self.description = bucket_payload.description
        self.url = bucket_payload.url
        self.schemaless = bucket_payload.schemaless
        self._buckets_router = buckets_router

    def __repr__(self):
        return f"Bucket<{self.name}>"

    def delete(self) -> Dict[str, Any]:
        return self._buckets_router.delete(self.uuid)

    def create_token(self, description: str = None) -> Dict[str, Any]:
        return self._buckets_router.create_token(self.uuid, description)

    def get_tokens(self) -> List[Dict[str, Any]]:
        return self._buckets_router.get_tokens(self.uuid)

    def delete_token(self, token_uuid: str) -> Dict[str, Any]:
        return self._buckets_router.delete_token(self.uuid, token_uuid)

    def push_data(self, rows: List[Dict[str, Any]]):
        return self._buckets_router.push_data(self.uuid, rows)

    def get_data(self) -> List[Dict[str, Any]]:
        return self._buckets_router.get_data(self.uuid)


class AdacordApi:
    """A facade to the Adacord API"""

    def __init__(self, client: HTTPClient = None):
        self.client = client or HTTPClient(auth=AccessTokenAuth(get_token))

    @property
    def User(self) -> User:
        return User(self.client)

    @property
    def Buckets(self) -> Buckets:
        return Buckets(self.client)

    @property
    def ApiTokens(self) -> Buckets:
        return ApiTokens(self.client)

    def Bucket(self, bucket_uuid: str) -> Bucket:
        return self.Buckets.get(bucket_uuid)

    @classmethod
    def Client(cls, with_auth: bool = True, token: str = None) -> "AdacordApi":
        if token:
            client = HTTPClient.with_token(token)
        elif with_auth:
            client = None
        else:
            client = HTTPClient()
        return cls(client)

    def create_bucket(self, description: str, schemaless: bool) -> Bucket:
        return self.Buckets.create(description, schemaless)

    def get_bucket(self, bucket_uuid: str) -> Bucket:
        return self.Buckets.get(bucket_uuid)


def create_api(with_auth: bool = True) -> AdacordApi:
    return AdacordApi.Client(with_auth=with_auth)
